@@ -14,6 +14,7 @@ ColumnDisplayFormatDialog::ColumnDisplayFormatDialog(const QString& colname, QSt
     ui->comboDisplayFormat->addItem(tr("Exponent notation"), "exponent");
     ui->comboDisplayFormat->addItem(tr("Hex blob"), "hexblob");
     ui->comboDisplayFormat->addItem(tr("Hex number"), "hex");
+    ui->comboDisplayFormat->addItem(tr("Apple NSDate to date"), "appleDate");
     ui->comboDisplayFormat->addItem(tr("Julian day to date"), "julian");
     ui->comboDisplayFormat->addItem(tr("Lower case"), "lower");
     ui->comboDisplayFormat->addItem(tr("Octal number"), "octal");
@@ -42,11 +43,7 @@ ColumnDisplayFormatDialog::~ColumnDisplayFormatDialog()
 
 QString ColumnDisplayFormatDialog::selectedDisplayFormat() const
 {
-#if QT_VERSION >= QT_VERSION_CHECK(5, 2, 0)
     if(ui->comboDisplayFormat->currentData().toString() == "default")
-#else
-    if(ui->comboDisplayFormat->itemData(ui->comboDisplayFormat->currentIndex()).toString() == "default")
-#endif
         return QString();
     else
         return ui->editDisplayFormat->text();
@@ -54,11 +51,8 @@ QString ColumnDisplayFormatDialog::selectedDisplayFormat() const
 
 void ColumnDisplayFormatDialog::updateSqlCode()
 {
-#if QT_VERSION >= QT_VERSION_CHECK(5, 2, 0)
     QString format = ui->comboDisplayFormat->currentData().toString();
-#else
-    QString format = ui->comboDisplayFormat->itemData(ui->comboDisplayFormat->currentIndex()).toString();
-#endif
+
     if(format == "default")
         ui->editDisplayFormat->setText(sqlb::escapeIdentifier(column_name));
     else if(format == "lower")
@@ -69,6 +63,8 @@ void ColumnDisplayFormatDialog::updateSqlCode()
         ui->editDisplayFormat->setText("datetime(" + sqlb::escapeIdentifier(column_name) + ", 'unixepoch')");
     else if(format == "winDate")
         ui->editDisplayFormat->setText("datetime ('1899-12-30', " + sqlb::escapeIdentifier(column_name) + " || \" days\" )");
+    else if(format == "appleDate")
+        ui->editDisplayFormat->setText("datetime ('2001-01-01', " + sqlb::escapeIdentifier(column_name) + " || \" seconds\" )");
     else if(format == "julian")
         ui->editDisplayFormat->setText("datetime(" + sqlb::escapeIdentifier(column_name) + ")");
     else if(format == "round")
