@@ -5,16 +5,17 @@
 
 class DBBrowserDB;
 class QTreeWidgetItem;
+namespace sqlb { class Object; typedef QSharedPointer<Object> ObjectPtr; }
 
 class DbStructureModel : public QAbstractItemModel
 {
     Q_OBJECT
 
 public:
-    explicit DbStructureModel(QObject* parent = 0);
+    explicit DbStructureModel(DBBrowserDB& db, QObject* parent = 0);
     ~DbStructureModel();
 
-    void reloadData(DBBrowserDB* db);
+    void reloadData();
 
     QVariant data(const QModelIndex& index, int role) const;
     Qt::ItemFlags flags(const QModelIndex& index) const;
@@ -30,7 +31,9 @@ public:
 
 private:
     QTreeWidgetItem* rootItem;
-    DBBrowserDB* m_db;
+    DBBrowserDB& m_db;
+
+    QTreeWidgetItem* addNode(QTreeWidgetItem* parent, const sqlb::ObjectPtr& object);
 };
 
 #endif
