@@ -23,23 +23,25 @@ public:
 public slots:
     virtual void setFocus();
     virtual void reject();
-    virtual void allowEditing(bool on);
+    void setReadOnly(bool ro);
+    void reloadSettings();
 
 protected:
     virtual void showEvent(QShowEvent* ev);
 
 private slots:
-    virtual void importData();
-    virtual void exportData();
-    virtual void setNull();
+    void importData();
+    void exportData();
+    void setNull();
+    void updateApplyButton();
     virtual void accept();
-    virtual int checkDataType(const QByteArray& data);
-    virtual void loadData(const QByteArray& data);
-    virtual void toggleOverwriteMode();
-    virtual void editModeChanged(int newMode);
-    virtual void editTextChanged();
-    virtual void updateCellInfo(const QByteArray& data);
-    virtual QString humanReadableSize(double byteCount);
+    int checkDataType(const QByteArray& data);
+    void loadData(const QByteArray& data);
+    void toggleOverwriteMode();
+    void editModeChanged(int newMode);
+    void editTextChanged();
+    void updateCellInfo(const QByteArray& data);
+    QString humanReadableSize(double byteCount);
 
 signals:
     void recordTextUpdated(const QPersistentModelIndex& idx, const QByteArray& data, bool isBlob);
@@ -51,6 +53,7 @@ private:
     int dataSource;
     int dataType;
     bool textNullSet;
+    bool isReadOnly;
 
     enum DataSources {
         TextBuffer,
