@@ -1,9 +1,6 @@
 #include "ExportSqlDialog.h"
 #include "ui_ExportSqlDialog.h"
 #include "sqlitedb.h"
-#include "PreferencesDialog.h"
-#include "sqlitetablemodel.h"
-#include "sqlite.h"
 #include "FileDialog.h"
 
 #include <QFile>
@@ -13,6 +10,14 @@
 static QString sSettingsGroup("exportsql");
 static QString sSettingsInsertColNames("insertcolnames");
 static QString sSettingsInsertMultiple("insertmultiple");
+static QString sSettingsOldSchema("oldschema");
+
+enum WhatComboEntries
+{
+    ExportEverything,
+    ExportSchemaOnly,
+    ExportDataOnly,
+};
 
 ExportSqlDialog::ExportSqlDialog(DBBrowserDB* db, QWidget* parent, const QString& selection)
     : QDialog(parent),
@@ -26,11 +31,12 @@ ExportSqlDialog::ExportSqlDialog(DBBrowserDB* db, QWidget* parent, const QString
     settings.beginGroup(sSettingsGroup);
     ui->checkColNames->setChecked(settings.value(sSettingsInsertColNames, false).toBool());
     ui->checkMultiple->setChecked(settings.value(sSettingsInsertMultiple, false).toBool());
+    ui->comboOldSchema->setCurrentIndex(settings.value(sSettingsOldSchema, 0).toInt());
 
     // Get list of tables to export
     objectMap objects = pdb->getBrowsableObjects();
-    for(objectMap::ConstIterator i=objects.begin();i!=objects.end();++i) {
-        ui->listTables->addItem(new QListWidgetItem(QIcon(QString(":icons/%1").arg(i.value().gettype())), i.value().getname()));
+    for(auto it=objects.constBegin();it!=objects.constEnd();++it) {
+        ui->listTables->addItem(new QListWidgetItem(QIcon(QString(":icons/%1").arg((*it)->type())), (*it)->name()));
     }
 
     // Sort list of tables and select the table specified in the
@@ -96,6 +102,7 @@ void ExportSqlDialog::accept()
     settings.beginGroup(sSettingsGroup);
     settings.setValue(sSettingsInsertColNames, ui->checkColNames->isChecked());
     settings.setValue(sSettingsInsertMultiple, ui->checkMultiple->isChecked());
+    settings.setValue(sSettingsOldSchema, ui->comboOldSchema->currentIndex());
     settings.endGroup();
 
     QStringList tables;
@@ -103,8 +110,9 @@ void ExportSqlDialog::accept()
         tables.push_back(item->text());
 
     // Check what to export. The indices here depend on the order of the items in the combobox in the ui file
-    bool exportSchema = ui->comboWhat->currentIndex() == 0 || ui->comboWhat->currentIndex() == 1;
-    bool exportData = ui->comboWhat->currentIndex() == 0 || ui->comboWhat->currentIndex() == 2;
+    bool exportSchema = ui->comboWhat->currentIndex() == ExportEverything || ui->comboWhat->currentIndex() == ExportSchemaOnly;
+    bool exportData = ui->comboWhat->currentIndex() == ExportEverything || ui->comboWhat->currentIndex() == ExportDataOnly;
+    bool keepSchema = ui->comboOldSchema->currentIndex() == 0;
 
     // Perform actual export
     bool dumpOk = pdb->dump(fileName,
@@ -112,7 +120,8 @@ void ExportSqlDialog::accept()
                             ui->checkColNames->isChecked(),
                             ui->checkMultiple->isChecked(),
                             exportSchema,
-                            exportData);
+                            exportData,
+                            keepSchema);
     if (dumpOk)
         QMessageBox::information(this, QApplication::applicationName(), tr("Export completed."));
     else
@@ -121,5 +130,12 @@ void ExportSqlDialog::accept()
     QDialog::accept();
 }
 
-
-
+void ExportSqlDialog::whatChanged(int index)
+{
+    // Only show the combobox for deciding what to do with the old schema when importing into an existing database when we're
+    // actually exporting the schema here
+    if(index != ExportDataOnly)
+        ui->comboOldSchema->setVisible(true);
+    else
+        ui->comboOldSchema->setVisible(false);
+}
