@@ -19,6 +19,9 @@ public:
 
 private slots:
     virtual void accept();
+    void doSelectAll();
+    void doDeselectAll();
+    void whatChanged(int index);
 
 private:
     Ui::ExportSqlDialog* ui;
