@@ -2,6 +2,7 @@
 #include "sqlitetablemodel.h"
 #include "FilterTableHeader.h"
 #include "sqlitetypes.h"
+#include "Settings.h"
 
 #include <QApplication>
 #include <QClipboard>
@@ -11,6 +12,7 @@
 #include <QHeaderView>
 #include <QMessageBox>
 #include <QBuffer>
+#include <QMenu>
 
 namespace
 {
@@ -31,7 +33,6 @@ QList<QStringList> parseClipboard(const QString& clipboard)
             // insert everything that left
             text = clipboard.mid(whitespace_offset);
             result.last().push_back(text);
-            offset = -1;
             break;
         }
 
@@ -65,6 +66,8 @@ ExtendedTableWidget::ExtendedTableWidget(QWidget* parent) :
     QTableView(parent)
 {
     setHorizontalScrollMode(ExtendedTableWidget::ScrollPerPixel);
+    // Force ScrollPerItem, so scrolling shows all table rows
+    setVerticalScrollMode(ExtendedTableWidget::ScrollPerItem);
 
     connect(verticalScrollBar(), SIGNAL(valueChanged(int)), this, SLOT(vscrollbarChanged(int)));
     connect(this, SIGNAL(clicked(QModelIndex)), this, SLOT(cellClicked(QModelIndex)));
@@ -73,7 +76,53 @@ ExtendedTableWidget::ExtendedTableWidget(QWidget* parent) :
     m_tableHeader = new FilterTableHeader(this);
     setHorizontalHeader(m_tableHeader);
 
+    // Set up vertical header context menu
     verticalHeader()->setContextMenuPolicy(Qt::CustomContextMenu);
+
+    // Set up table view context menu
+    m_contextMenu = new QMenu(this);
+    QAction* nullAction = new QAction(tr("Set to NULL"), m_contextMenu);
+    QAction* copyAction = new QAction(QIcon(":/icons/copy"), tr("Copy"), m_contextMenu);
+    QAction* pasteAction = new QAction(QIcon(":/icons/paste"), tr("Paste"), m_contextMenu);
+    m_contextMenu->addAction(nullAction);
+    m_contextMenu->addSeparator();
+    m_contextMenu->addAction(copyAction);
+    m_contextMenu->addAction(pasteAction);
+    setContextMenuPolicy(Qt::CustomContextMenu);
+
+    // Set up context menu actions
+    connect(this, static_cast<void(QTableView::*)(const QPoint&)>(&QTableView::customContextMenuRequested),
+            [=](const QPoint& pos)
+    {
+        // Try to find out whether the current view is editable and (de)activate menu options according to that
+        bool editable = editTriggers() != QAbstractItemView::NoEditTriggers;
+        nullAction->setEnabled(editable);
+        pasteAction->setEnabled(editable);
+
+        // Show menu
+        m_contextMenu->popup(viewport()->mapToGlobal(pos));
+    });
+    connect(nullAction, &QAction::triggered, [&]() {
+        foreach(const QModelIndex& index, selectedIndexes())
+            model()->setData(index, QVariant());
+    });
+    connect(copyAction, &QAction::triggered, [&]() {
+       copy();
+    });
+    connect(pasteAction, &QAction::triggered, [&]() {
+       paste();
+    });
+}
+
+void ExtendedTableWidget::reloadSettings()
+{
+    // Set the new font and font size
+    QFont dataBrowserFont(Settings::getValue("databrowser", "font").toString());
+    dataBrowserFont.setPointSize(Settings::getValue("databrowser", "fontsize").toInt());
+    setFont(dataBrowserFont);
+
+    // Set new default row height depending on the font size
+    verticalHeader()->setDefaultSectionSize(verticalHeader()->fontMetrics().height()+10);
 }
 
 void ExtendedTableWidget::copy()
@@ -87,6 +136,8 @@ void ExtendedTableWidget::copy()
 
     SqliteTableModel* m = qobject_cast<SqliteTableModel*>(model());
 
+    m_buffer.clear();
+
     // If a single cell is selected, copy it to clipboard
     if (indices.size() == 1) {
         QImage img;
@@ -96,13 +147,23 @@ void ExtendedTableWidget::copy()
             qApp->clipboard()->setImage(img);
             return;
         } else {
-            qApp->clipboard()->setText(data.toString());
+            QString text = data.toString();
+            if (text.isEmpty()) {
+                // NULL and empty single-cells are handled via inner buffer
+                qApp->clipboard()->clear();
+                QByteArrayList lst;
+                lst << data.toByteArray();
+                m_buffer.push_back(lst);
+                return;
+            }
+
+            if (text.contains('\n'))
+                text = QString("\"%1\"").arg(text);
+            qApp->clipboard()->setText(text);
             return;
         }
     }
 
-    m_buffer.clear();
-
     // If any of the cells contain binary data - we use inner buffer
     bool containsBinary = false;
     foreach (const QModelIndex& index, indices)
@@ -232,7 +293,7 @@ void ExtendedTableWidget::paste()
         {
             // Ask user is it sure about this
             QMessageBox::StandardButton reply = QMessageBox::question(this, QApplication::applicationName(),
-                tr("The content of clipboard is bigger than the range selected.\nDo you want to insert it anyway?"),
+                tr("The content of the clipboard is bigger than the range selected.\nDo you want to insert it anyway?"),
                 QMessageBox::Yes|QMessageBox::No);
             if(reply != QMessageBox::Yes)
             {
@@ -287,9 +348,8 @@ void ExtendedTableWidget::keyPressEvent(QKeyEvent* event)
     {
         copy();
         return;
-    // Call a custom paste method when Ctrl-P is pressed
-    } else if(event->matches(QKeySequence::Paste))
-    {
+    } else if(event->matches(QKeySequence::Paste)) {
+        // Call a custom paste method when Ctrl-P is pressed
         paste();
     } else if(event->key() == Qt::Key_Tab && hasFocus() &&
               selectedIndexes().count() == 1 &&
@@ -307,6 +367,10 @@ void ExtendedTableWidget::keyPressEvent(QKeyEvent* event)
             foreach(const QModelIndex& index, selectedIndexes())
                 model()->setData(index, "");
         }
+    } else if(event->modifiers().testFlag(Qt::ControlModifier) && (event->key() == Qt::Key_PageUp || event->key() == Qt::Key_PageDown)) {
+        // When pressing Ctrl + Page up/down send a signal indicating the user wants to change the current table
+        emit switchTable(event->key() == Qt::Key_PageDown);
+        return;
     }
 
     // This prevents the current selection from being changed when pressing tab to move to the next filter. Note that this is in an 'if' condition,
@@ -362,7 +426,7 @@ QSet<int> ExtendedTableWidget::selectedCols()
 
 void ExtendedTableWidget::cellClicked(const QModelIndex& index)
 {
-    // If Alt key is pressed try to jump to the row referenced by the foreign key of the clicked cell
+    // If Ctrl-Shift is pressed try to jump to the row referenced by the foreign key of the clicked cell
     if(qApp->keyboardModifiers().testFlag(Qt::ControlModifier) && qApp->keyboardModifiers().testFlag(Qt::ShiftModifier) && model())
     {
         SqliteTableModel* m = qobject_cast<SqliteTableModel*>(model());
