@@ -2,11 +2,13 @@
 #define EXTENDEDTABLEWIDGET_H
 
 #include <QTableView>
-#include "FilterTableHeader.h"
 #include <QSet>
 #include <QDropEvent>
 #include <QDragMoveEvent>
 
+class QMenu;
+class FilterTableHeader;
+
 class ExtendedTableWidget : public QTableView
 {
     Q_OBJECT
@@ -18,14 +20,18 @@ public:
 
 public:
     QSet<int> selectedCols();
+    int numVisibleRows();
+
+public slots:
+    void reloadSettings();
 
 signals:
     void foreignKeyClicked(const QString& table, const QString& column, const QByteArray& value);
+    void switchTable(bool next);    // 'next' parameter is set to true if next table should be selected and to false if previous table should be selected
 
 private:
     void copy();
     void paste();
-    int numVisibleRows();
 
     typedef QList<QByteArray> QByteArrayList;
     QList<QByteArrayList> m_buffer;
@@ -42,6 +48,7 @@ protected:
     virtual void dropEvent(QDropEvent* event);
 
     FilterTableHeader* m_tableHeader;
+    QMenu* m_contextMenu;
 };
 
 #endif
