@@ -1,5 +1,5 @@
 #include "FileDialog.h"
-#include "PreferencesDialog.h"
+#include "Settings.h"
 
 QString FileDialog::getOpenFileName(QWidget* parent, const QString& caption, const QString &filter, QString *selectedFilter, Options options)
 {
@@ -9,13 +9,24 @@ QString FileDialog::getOpenFileName(QWidget* parent, const QString& caption, con
     return result;
 }
 
+QStringList FileDialog::getOpenFileNames(QWidget *parent, const QString &caption, const QString &filter, QString *selectedFilter, QFileDialog::Options options)
+{
+    QStringList result = QFileDialog::getOpenFileNames(parent, caption, getFileDialogPath(), filter, selectedFilter, options);
+    if(!result.isEmpty())
+    {
+        QFileInfo path = QFileInfo(result.first());
+        setFileDialogPath(path.absolutePath());
+    }
+    return result;
+}
+
 QString FileDialog::getSaveFileName(QWidget* parent, const QString& caption, const QString& filter, const QString& defaultFileName, QString* selectedFilter, Options options)
 {
     QString dir = getFileDialogPath();
     if(!defaultFileName.isEmpty())
         dir += "/" + defaultFileName;
 
-    QString result = QFileDialog::getSaveFileName(parent, caption, defaultFileName, filter, selectedFilter, options);
+    QString result = QFileDialog::getSaveFileName(parent, caption, dir, filter, selectedFilter, options);
     if(!result.isEmpty())
         setFileDialogPath(result);
     return result;
@@ -31,13 +42,13 @@ QString FileDialog::getExistingDirectory(QWidget* parent, const QString& caption
 
 QString FileDialog::getFileDialogPath()
 {
-    switch(PreferencesDialog::getSettingsValue("db", "savedefaultlocation").toInt())
+    switch(Settings::getValue("db", "savedefaultlocation").toInt())
     {
     case 0:     // Remember last location
     case 2:     // Remember last location for current session only
-        return PreferencesDialog::getSettingsValue("db", "lastlocation").toString();
+        return Settings::getValue("db", "lastlocation").toString();
     case 1:     // Always use this locations
-        return PreferencesDialog::getSettingsValue("db", "defaultlocation").toString();
+        return Settings::getValue("db", "defaultlocation").toString();
     default:
         return "";
     }
@@ -47,13 +58,13 @@ void FileDialog::setFileDialogPath(const QString& new_path)
 {
     QString dir = QFileInfo(new_path).absolutePath();
 
-    switch(PreferencesDialog::getSettingsValue("db", "savedefaultlocation").toInt())
+    switch(Settings::getValue("db", "savedefaultlocation").toInt())
     {
     case 0:     // Remember last location
-        PreferencesDialog::setSettingsValue("db", "lastlocation", dir);
+        Settings::setValue("db", "lastlocation", dir);
         break;
     case 2:     // Remember last location for current session only
-        PreferencesDialog::setSettingsValue("db", "lastlocation", dir, true);
+        Settings::setValue("db", "lastlocation", dir, true);
         break;
     case 1:     // Always use this locations
         break;  // Do nothing
