@@ -5,10 +5,15 @@
 
 class FileDialog : public QFileDialog
 {
+    Q_OBJECT
+
 public:
     static QString getOpenFileName(QWidget* parent = 0, const QString& caption = QString(),
                                    const QString& filter = QString(), QString* selectedFilter = 0,
                                    Options options = 0);
+    static QStringList getOpenFileNames(QWidget* parent = 0, const QString& caption = QString(),
+                                        const QString& filter = QString(), QString* selectedFilter = 0,
+                                        Options options = 0);
     static QString getSaveFileName(QWidget* parent = 0, const QString& caption = QString(),
                                    const QString& filter = QString(), const QString& defaultFileName = QString(), QString* selectedFilter = 0,
                                    Options options = 0);
@@ -17,7 +22,7 @@ public:
 
     static QString getSqlDatabaseFileFilter()
     {
-        return tr("SQLite database files (*.db *.sqlite *.sqlite3 *.db3);;All files (*)");
+        return QObject::tr("SQLite database files (*.db *.sqlite *.sqlite3 *.db3);;All files (*)");
     }
 
 private:
