@@ -17,9 +17,10 @@ public:
     virtual QSize sizeHint() const;
     
 public slots:
-    void generateFilters(int number, bool bKeepValues = false);
+    void generateFilters(int number, bool showFirst = false);
     void adjustPositions();
     void clearFilters();
+    void setFilter(int column, const QString& value);
 
 signals:
     void filterChanged(int column, QString value);
