@@ -18,16 +18,17 @@
 #include <QFileInfo>
 #include <memory>
 
-ImportCsvDialog::ImportCsvDialog(const QString& filename, DBBrowserDB* db, QWidget* parent)
+ImportCsvDialog::ImportCsvDialog(const QStringList &filenames, DBBrowserDB* db, QWidget* parent)
     : QDialog(parent),
       ui(new Ui::ImportCsvDialog),
-      csvFilename(filename),
+      csvFilenames(filenames),
       pdb(db)
 {
     ui->setupUi(this);
 
     // Get the actual file name out of the provided path and use it as the default table name for import
-    QFileInfo file(filename);
+    // For importing several files at once, the fields have to be the same so we can safely use the first
+    QFileInfo file(filenames.first());
     ui->editName->setText(file.baseName());
 
     // Create a list of all available encodings and create an auto completion list from them
@@ -42,13 +43,29 @@ ImportCsvDialog::ImportCsvDialog(const QString& filename, DBBrowserDB* db, QWidg
     QSettings settings(QApplication::organizationName(), QApplication::organizationName());
     ui->checkboxHeader->setChecked(settings.value("importcsv/firstrowheader", false).toBool());
     ui->checkBoxTrimFields->setChecked(settings.value("importcsv/trimfields", true).toBool());
+    ui->checkBoxSeparateTables->setChecked(settings.value("importcsv/separatetables", false).toBool());
     setSeparatorChar(QChar(settings.value("importcsv/separator", ',').toInt()));
     setQuoteChar(QChar(settings.value("importcsv/quotecharacter", '"').toInt()));
     setEncoding(settings.value("importcsv/encoding", "UTF-8").toString());
 
-    // Initialise user interface
-    checkInput();
+    // Prepare and show interface depending on how many files are selected
+    if (csvFilenames.length() > 1)
+    {
+        ui->separateTables->setVisible(true);
+        ui->checkBoxSeparateTables->setVisible(true);
+        ui->filePickerBlock->setVisible(true);
+        selectFiles();
+    }
+    else if (csvFilenames.length() == 1)
+    {
+        ui->separateTables->setVisible(false);
+        ui->checkBoxSeparateTables->setVisible(false);
+        ui->filePickerBlock->setVisible(false);
+    }
+
+    selectedFile = csvFilenames.first();
     updatePreview();
+    checkInput();
 }
 
 ImportCsvDialog::~ImportCsvDialog()
@@ -128,132 +145,23 @@ void ImportCsvDialog::accept()
     settings.setValue("separator", currentSeparatorChar());
     settings.setValue("quotecharacter", currentQuoteChar());
     settings.setValue("trimfields", ui->checkBoxTrimFields->isChecked());
+    settings.setValue("separatetables", ui->checkBoxSeparateTables->isChecked());
     settings.setValue("encoding", currentEncoding());
     settings.endGroup();
 
-    // Parse all csv data
-    QFile file(csvFilename);
-    file.open(QIODevice::ReadOnly);
-
-    CSVParser csv(ui->checkBoxTrimFields->isChecked(), currentSeparatorChar(), currentQuoteChar());
-    csv.setCSVProgress(new CSVImportProgress(file.size()));
-
-    QTextStream tstream(&file);
-    tstream.setCodec(currentEncoding().toUtf8());
-    csv.parse(tstream);
-    file.close();
-
-    if(csv.csv().size() == 0)
-        return;
-
-    // Generate field names. These are either taken from the first CSV row or are generated in the format of "fieldXY" depending on the user input
-    sqlb::FieldVector fieldList;
-    CSVParser::TCSVResult::const_iterator itBegin = csv.csv().begin();
-    if(ui->checkboxHeader->isChecked())
+    // Get all the selected files and start the import
+    if (ui->filePickerBlock->isVisible())
     {
-        ++itBegin;
-        for(QStringList::const_iterator it = csv.csv().at(0).begin();
-            it != csv.csv().at(0).end();
-            ++it)
-        {
-            // Remove invalid characters
-            QString thisfield = *it;
-            thisfield.replace("`", "");
-            thisfield.replace(" ", "");
-            thisfield.replace('"', "");
-            thisfield.replace("'","");
-            thisfield.replace(",","");
-            thisfield.replace(";","");
-
-            // Avoid empty field names
-            if(thisfield.isEmpty())
-                thisfield = QString("field%1").arg(std::distance(csv.csv().at(0).begin(), it) + 1);
-
-            fieldList.push_back(sqlb::FieldPtr(new sqlb::Field(thisfield, "")));
-        }
-    } else {
-        for(size_t i=0; i < csv.columns(); ++i)
-            fieldList.push_back(sqlb::FieldPtr(new sqlb::Field(QString("field%1").arg(i+1), "")));
-    }
-
-    // Show progress dialog
-    QProgressDialog progress(tr("Inserting data..."), tr("Cancel"), 0, csv.csv().size());
-    progress.setWindowModality(Qt::ApplicationModal);
-    progress.show();
-
-    // Are we importing into an existing table?
-    bool importToExistingTable = false;
-    objectMap objects = pdb->getBrowsableObjects();
-    for(objectMap::ConstIterator i=objects.begin();i!=objects.end();++i)
-    {
-        if(i.value().gettype() == "table" && i.value().getname() == ui->editName->text())
-        {
-            if((size_t)i.value().table.fields().size() != csv.columns())
-            {
-                QMessageBox::warning(this, QApplication::applicationName(),
-                                     tr("There is already a table of that name and an import into an existing table is only possible if the number of columns match."));
-                return;
-            } else {
-                if(QMessageBox::question(this, QApplication::applicationName(), tr("There is already a table of that name. Do you want to import the data into it?"), QMessageBox::Yes, QMessageBox::No) == QMessageBox::Yes)
-                {
-                    importToExistingTable = true;
-                    break;
-                } else {
-                    return;
-                }
+        for (int i = 0; i < ui->filePicker->count(); i++) {
+            auto item = ui->filePicker->item(i);
+            if (item->checkState() == Qt::Checked) {
+                importCsv(item->data(Qt::DisplayRole).toString(), item->data(Qt::UserRole).toString());
             }
         }
     }
-
-    // Create a savepoint, so we can rollback in case of any errors during importing
-    // db needs to be saved or an error will occur
-    QString restorepointName = QString("CSVIMPORT_%1").arg(QDateTime::currentMSecsSinceEpoch());
-    if(!pdb->setSavepoint(restorepointName))
-        return rollback(this, pdb, progress, restorepointName, 0, tr("Creating restore point failed: %1").arg(pdb->lastErrorMessage));
-
-    // Create table
-    if(!importToExistingTable)
+    else
     {
-        if(!pdb->createTable(ui->editName->text(), fieldList))
-            return rollback(this, pdb, progress, restorepointName, 0, tr("Creating the table failed: %1").arg(pdb->lastErrorMessage));
-    }
-
-    // now lets import all data, one row at a time
-    for(CSVParser::TCSVResult::const_iterator it = itBegin;
-        it != csv.csv().end();
-        ++it)
-    {
-        QString sql = QString("INSERT INTO %1 VALUES(").arg(sqlb::escapeIdentifier(ui->editName->text()));
-
-        QStringList insertlist;
-        for(QStringList::const_iterator jt = it->begin(); jt != it->end(); ++jt)
-        {
-            // need to mprintf here
-            char* formSQL = sqlite3_mprintf("%Q", (const char*)jt->toUtf8());
-            insertlist << formSQL;
-            if(formSQL)
-                sqlite3_free(formSQL);
-        }
-
-        // add missing fields with empty values
-        for(unsigned int i = insertlist.size(); i < csv.columns(); ++i)
-        {
-            qWarning() << "ImportCSV" << tr("Missing field for record %1").arg(std::distance(itBegin, it) + 1);
-            insertlist << "NULL";
-        }
-
-        sql.append(insertlist.join(QChar(',')));
-        sql.append(");");
-
-        if(!pdb->executeSQL(sql, false, false))
-            return rollback(this, pdb, progress, restorepointName, std::distance(itBegin, it) + 1, tr("Inserting row failed: %1").arg(pdb->lastErrorMessage));
-
-        // Update progress bar and check if cancel button was clicked
-        unsigned int prog = std::distance(csv.csv().begin(), it);
-        if(prog % 100 == 0)
-            progress.setValue(prog);
-        if(progress.wasCanceled())
-            return rollback(this, pdb, progress, restorepointName, std::distance(itBegin, it) + 1, "");
+        importCsv(csvFilenames.first());
     }
 
     QApplication::restoreOverrideCursor();  // restore original cursor
@@ -268,7 +176,7 @@ void ImportCsvDialog::updatePreview()
     ui->editCustomEncoding->setVisible(ui->comboEncoding->currentIndex() == ui->comboEncoding->count()-1);
 
     // Get preview data
-    QFile file(csvFilename);
+    QFile file(selectedFile);
     file.open(QIODevice::ReadOnly);
 
     CSVParser csv(ui->checkBoxTrimFields->isChecked(), currentSeparatorChar(), currentQuoteChar());
@@ -320,11 +228,279 @@ void ImportCsvDialog::updatePreview()
 
 void ImportCsvDialog::checkInput()
 {
-    bool valid = true;
-    if(ui->editName->text().isEmpty())
-        valid = false;
+    bool allowImporting = false;
+    if (ui->filePickerBlock->isVisible()) {
+        bool checkedItem = false;
+        for (int i = 0; i < ui->filePicker->count(); i++) {
+            if (ui->filePicker->item(i)->checkState() == Qt::Checked) checkedItem = true;
+        }
+        allowImporting = !ui->editName->text().isEmpty() && checkedItem;
+    } else {
+        allowImporting = !ui->editName->text().isEmpty();
+    }
 
-    ui->buttonBox->button(QDialogButtonBox::Ok)->setEnabled(valid);
+    if (ui->filePicker->currentItem()) {
+        ui->filePicker->currentItem()->setData(Qt::UserRole, ui->editName->text());
+    }
+
+    ui->matchSimilar->setEnabled(ui->filePicker->currentItem() != nullptr);
+    ui->buttonBox->button(QDialogButtonBox::Ok)->setEnabled(allowImporting);
+}
+
+void ImportCsvDialog::selectFiles()
+{
+    for (auto fileName : csvFilenames) {
+        auto fInfo = QFileInfo(fileName);
+        auto item = new QListWidgetItem();
+        item->setText(fileName);
+        item->setData(Qt::UserRole, fInfo.baseName());
+        item->setCheckState(Qt::Checked);
+        ui->filePicker->addItem(item);
+    }
+
+    connect(ui->filePicker, SIGNAL(itemClicked(QListWidgetItem*)), SLOT(updateSelectedFilePreview(QListWidgetItem*)));
+}
+
+void ImportCsvDialog::updateSelectedFilePreview(QListWidgetItem* item)
+{
+    selectedFile = item->data(Qt::DisplayRole).toString();
+    QFileInfo fileInfo(selectedFile);
+    if (ui->checkBoxSeparateTables->isChecked()) {
+        ui->editName->setText(item->data(Qt::UserRole).toString());
+    } else {
+        ui->editName->setText(fileInfo.baseName());
+    }
+    updatePreview();
+    checkInput();
+}
+
+void ImportCsvDialog::updateSelection(bool selected)
+{
+    for (int i = 0; i < ui->filePicker->count(); i++)
+        ui->filePicker->item(i)->setCheckState(selected ? Qt::Checked : Qt::Unchecked);
+    ui->toggleSelected->setText(selected ? tr("Deselect All") : tr("Select All"));
+    checkInput();
+}
+
+void ImportCsvDialog::matchSimilar()
+{
+    auto item = ui->filePicker->currentItem();
+    auto selectedHeader = generateFieldList(parseCSV(item->data(Qt::DisplayRole).toString(), 1));
+
+    for (int i = 0; i < ui->filePicker->count(); i++)
+    {
+        auto item = ui->filePicker->item(i);
+        auto header = generateFieldList(parseCSV(item->data(Qt::DisplayRole).toString(), 1));
+        bool matchingHeader = false;
+
+        if (selectedHeader.count() == header.count())
+        {
+            matchingHeader = std::equal(selectedHeader.begin(), selectedHeader.end(), header.begin(),
+                                        [](const sqlb::FieldPtr& item1, const sqlb::FieldPtr& item2) -> bool {
+                                            return (item1->name() == item2->name());
+                                        });
+            if (matchingHeader) {
+                item->setCheckState(Qt::Checked);
+                item->setBackgroundColor(Qt::green);
+            }
+        }
+        else
+        {
+            item->setCheckState(Qt::Unchecked);
+            item->setBackground(Qt::white);
+        }
+    }
+
+    checkInput();
+}
+
+CSVParser ImportCsvDialog::parseCSV(const QString &fileName, qint64 count)
+{
+    // Parse all csv data
+    QFile file(fileName);
+    file.open(QIODevice::ReadOnly);
+
+    CSVParser csv(ui->checkBoxTrimFields->isChecked(), currentSeparatorChar(), currentQuoteChar());
+    // If count is one, we only want the header, no need to see progress
+    if (count != 1) csv.setCSVProgress(new CSVImportProgress(file.size()));
+
+    QTextStream tstream(&file);
+    tstream.setCodec(currentEncoding().toUtf8());
+    csv.parse(tstream, count);
+    file.close();
+
+    return csv;
+}
+
+sqlb::FieldVector ImportCsvDialog::generateFieldList(const CSVParser &parser)
+{
+    if (parser.csv().size() == 0) return sqlb::FieldVector();
+
+    // Generate field names. These are either taken from the first CSV row or are generated in the format of "fieldXY" depending on the user input
+    sqlb::FieldVector fieldList;
+    if(ui->checkboxHeader->isChecked())
+    {
+        for(QStringList::const_iterator it = parser.csv().at(0).begin();
+            it != parser.csv().at(0).end();
+            ++it)
+        {
+            // Remove invalid characters
+            QString thisfield = *it;
+            thisfield.replace("`", "");
+            thisfield.replace(" ", "");
+            thisfield.replace('"', "");
+            thisfield.replace("'","");
+            thisfield.replace(",","");
+            thisfield.replace(";","");
+
+            // Avoid empty field names
+            if(thisfield.isEmpty())
+                thisfield = QString("field%1").arg(std::distance(parser.csv().at(0).begin(), it) + 1);
+
+            fieldList.push_back(sqlb::FieldPtr(new sqlb::Field(thisfield, "")));
+        }
+    } else {
+        for(size_t i=0; i < parser.columns(); ++i)
+            fieldList.push_back(sqlb::FieldPtr(new sqlb::Field(QString("field%1").arg(i+1), "")));
+    }
+
+    return fieldList;
+}
+
+void ImportCsvDialog::importCsv(const QString& fileName, const QString &name)
+{
+    QString tableName;
+
+    if (csvFilenames.size() > 1 && ui->checkBoxSeparateTables->isChecked()) {
+        if (name.isEmpty()) {
+            QFileInfo fileInfo(fileName);
+            tableName = fileInfo.baseName();
+        } else {
+            tableName = name;
+        }
+    } else {
+        tableName = ui->editName->text();
+    }
+
+    CSVParser csv = parseCSV(fileName);
+    if (csv.csv().size() == 0)  return;
+
+    sqlb::FieldVector fieldList = generateFieldList(csv);
+
+    // Show progress dialog
+    QProgressDialog progress(tr("Inserting data..."), tr("Cancel"), 0, csv.csv().size());
+    progress.setWindowModality(Qt::ApplicationModal);
+    progress.show();
+
+    // Are we importing into an existing table?
+    bool importToExistingTable = false;
+    const sqlb::ObjectPtr obj = pdb->getObjectByName(tableName);
+    if(obj && obj->type() == sqlb::Object::Types::Table)
+    {
+        if((size_t)obj.dynamicCast<sqlb::Table>()->fields().size() != csv.columns())
+        {
+            QMessageBox::warning(this, QApplication::applicationName(),
+                                 tr("There is already a table of that name and an import into an existing table is only possible if the number of columns match."));
+            return;
+        } else {
+            // If we are importing multiple files, we can skip the warning and perform the inserts
+            // To omit further warnings there is a filter button to select all conforming sql
+            if (!ui->checkBoxSeparateTables->isChecked())
+                importToExistingTable = true;
+
+            if(QMessageBox::question(this, QApplication::applicationName(), tr("There is already a table of that name. Do you want to import the data into it?"), QMessageBox::Yes, QMessageBox::No) == QMessageBox::Yes)
+                importToExistingTable = true;
+            else
+                return;
+        }
+    }
+
+    // Create a savepoint, so we can rollback in case of any errors during importing
+    // db needs to be saved or an error will occur
+    QString restorepointName = pdb->generateSavepointName("csvimport");
+    if(!pdb->setSavepoint(restorepointName))
+        return rollback(this, pdb, progress, restorepointName, 0, tr("Creating restore point failed: %1").arg(pdb->lastError()));
+
+    // Create table
+    QStringList nullValues;
+    if(!importToExistingTable)
+    {
+        if(!pdb->createTable(tableName, fieldList))
+            return rollback(this, pdb, progress, restorepointName, 0, tr("Creating the table failed: %1").arg(pdb->lastError()));
+    } else {
+        // Importing into an existing table. So find out something about it's structure.
+
+        // Prepare the values for each table column that are to be inserted if the field in the CSV file is empty. Depending on the data type
+        // and the constraints of a field, we need to handle this case differently.
+        sqlb::TablePtr tbl = pdb->getObjectByName(tableName).dynamicCast<sqlb::Table>();
+        if(tbl)
+        {
+            foreach(const sqlb::FieldPtr& f, tbl->fields())
+            {
+                if(f->isInteger() && f->notnull())              // If this is an integer column but NULL isn't allowed, insert 0
+                    nullValues << "0";
+                else if(f->isInteger() && !f->notnull())        // If this is an integer column and NULL is allowed, insert NULL
+                    nullValues << QString();
+                else                                            // Otherwise (i.e. if this isn't an integer column), insert an empty string
+                    nullValues << "''";
+            }
+        }
+    }
+
+    // now lets import all data, one row at a time
+    CSVParser::TCSVResult::const_iterator itBegin = csv.csv().begin();
+    if(ui->checkboxHeader->isChecked())     // If the first row contains the field names we should skip it here because this is the data import
+        ++itBegin;
+    for(CSVParser::TCSVResult::const_iterator it = itBegin;
+        it != csv.csv().end();
+        ++it)
+    {
+        QString sql = QString("INSERT INTO %1 VALUES(").arg(sqlb::escapeIdentifier(tableName));
+
+        QStringList insertlist;
+        for(int i=0;i<it->size();i++)
+        {
+            // Empty values need special treatment, but only when importing into an existing table where we could find out something about
+            // its table definition
+            if(importToExistingTable && it->at(i).isEmpty() && nullValues.size() > i)
+            {
+                // This is an empty value. We'll need to look up how to handle it depending on the field to be inserted into.
+                QString val = nullValues.at(i);
+                if(val.isNull())
+                    insertlist << "NULL";
+                else
+                    insertlist << val;
+            } else {
+                // This is a non-empty value. Just add it to the list. The sqlite3_mprintf call with the %Q placeholder takes the value,
+                // adds single quotes around it and doubles all single quotes inside it. This means it'll be safe to be inserted into the SQL
+                // statement.
+                char* formSQL = sqlite3_mprintf("%Q", (const char*)it->at(i).toUtf8());
+                insertlist << formSQL;
+                if(formSQL)
+                    sqlite3_free(formSQL);
+            }
+        }
+
+        // add missing fields with empty values
+        for(unsigned int i = insertlist.size(); i < csv.columns(); ++i)
+        {
+            qWarning() << "ImportCSV" << tr("Missing field for record %1").arg(std::distance(itBegin, it) + 1);
+            insertlist << "NULL";
+        }
+
+        sql.append(insertlist.join(QChar(',')));
+        sql.append(");");
+
+        if(!pdb->executeSQL(sql, false, false))
+            return rollback(this, pdb, progress, restorepointName, std::distance(itBegin, it) + 1, tr("Inserting row failed: %1").arg(pdb->lastError()));
+
+        // Update progress bar and check if cancel button was clicked
+        unsigned int prog = std::distance(csv.csv().begin(), it);
+        if(prog % 100 == 0)
+            progress.setValue(prog);
+        if(progress.wasCanceled())
+            return rollback(this, pdb, progress, restorepointName, std::distance(itBegin, it) + 1, "");
+    }
 }
 
 void ImportCsvDialog::setQuoteChar(const QChar& c)
