@@ -7,6 +7,8 @@
 
 class QTreeWidgetItem;
 class QFrame;
+class QTableWidget;
+class QSslCertificate;
 
 namespace Ui {
 class PreferencesDialog;
@@ -20,10 +22,6 @@ public:
     explicit PreferencesDialog(QWidget* parent = 0);
     ~PreferencesDialog();
 
-    // Use these methods to access the application settings.
-    static QVariant getSettingsValue(const QString& group, const QString& name);
-    static void setSettingsValue(const QString& group, const QString& name, const QVariant& value, bool dont_save_to_disk = false);
-
 private slots:
     virtual void loadSettings();
     virtual void saveSettings();
@@ -32,19 +30,18 @@ private slots:
     virtual void showColourDialog(QTreeWidgetItem* item, int column);
     virtual void addExtension();
     virtual void removeExtension();
+    virtual void activateRemoteTab(bool active);
+    virtual void addClientCertificate();
+    virtual void removeClientCertificate();
+    void chooseRemoteCloneDirectory();
 
 private:
     Ui::PreferencesDialog *ui;
 
-    // This works similar to getSettingsValue but returns the default value instead of the value set by the user
-    static QVariant getSettingsDefaultValue(const QString& group, const QString& name);
-
-    // Cache for storing the settings to avoid repeatedly reading the settings file all the time
-    static QHash<QString, QVariant> m_hCache;
-
     void fillLanguageBox();
     void loadColorSetting(QFrame *frame, const QString &name);
     void saveColorSetting(QFrame *frame, const QString &name);
+    void addClientCertToTable(const QString& path, const QSslCertificate& cert);
 
 protected:
     bool eventFilter(QObject *obj, QEvent *event);
