@@ -1,17 +1,13 @@
 #ifndef SQLEXECUTIONAREA_H
 #define SQLEXECUTIONAREA_H
 
-#include "sqltextedit.h"
-
 #include <QWidget>
 
-#include "ExtendedTableWidget.h"
-
 class SqlTextEdit;
 class SqliteTableModel;
 class DBBrowserDB;
 class QMenu;
-class QTextEdit;
+class ExtendedTableWidget;
 
 namespace Ui {
 class SqlExecutionArea;
@@ -22,7 +18,7 @@ class SqlExecutionArea : public QWidget
     Q_OBJECT
 
 public:
-    explicit SqlExecutionArea(QWidget* parent, DBBrowserDB* _db);
+    explicit SqlExecutionArea(DBBrowserDB& _db, QWidget* parent = Q_NULLPTR);
     ~SqlExecutionArea();
 
     QString getSql() const;
@@ -32,7 +28,6 @@ public:
     void setFileName(const QString& filename) { sqlFileName = filename; }
 
     SqliteTableModel* getModel() { return model; }
-    QTextEdit* getResultView();
     SqlTextEdit* getEditor();
     ExtendedTableWidget *getTableResult();
     
@@ -44,7 +39,7 @@ public slots:
     virtual void reloadSettings();
 
 private:
-    DBBrowserDB* db;
+    DBBrowserDB& db;
     SqliteTableModel* model;
     QMenu* menuPopupSave;
     QString sqlFileName;
