@@ -44,21 +44,22 @@ public:
 	public: void statementlist();
 	public: void statement();
 	public: void createtable();
+	public: void createindex();
 	public: void create_statements();
 	public: void keywordastablename();
 	public: void columndef();
 	public: void tableconstraint();
 	public: void selectstmt();
+	public: void name();
+	public: void expr();
+	public: void indexedcolumn();
 	public: void keywordascolumnname();
 	public: void columnname();
 	public: void type_name();
 	public: void columnconstraint();
-	public: void name();
 	public: void conflictclause();
-	public: void expr();
 	public: void literalvalue();
 	public: void foreignkeyclause();
-	public: void indexedcolumn();
 	public: void functionname();
 	public: void subexpr();
 	public: void binaryoperator();
@@ -79,10 +80,10 @@ protected:
 private:
 	static const char* tokenNames[];
 #ifndef NO_STATIC_CONSTS
-	static const int NUM_TOKENS = 107;
+	static const int NUM_TOKENS = 113;
 #else
 	enum {
-		NUM_TOKENS = 107
+		NUM_TOKENS = 113
 	};
 #endif
 	
@@ -128,6 +129,12 @@ private:
 	static const ANTLR_USE_NAMESPACE(antlr)BitSet _tokenSet_19;
 	static const unsigned long _tokenSet_20_data_[];
 	static const ANTLR_USE_NAMESPACE(antlr)BitSet _tokenSet_20;
+	static const unsigned long _tokenSet_21_data_[];
+	static const ANTLR_USE_NAMESPACE(antlr)BitSet _tokenSet_21;
+	static const unsigned long _tokenSet_22_data_[];
+	static const ANTLR_USE_NAMESPACE(antlr)BitSet _tokenSet_22;
+	static const unsigned long _tokenSet_23_data_[];
+	static const ANTLR_USE_NAMESPACE(antlr)BitSet _tokenSet_23;
 };
 
 #endif /*INC_Sqlite3Parser_hpp_*/
