@@ -16,36 +16,16 @@ enum
     kLogMsg_App
 };
 
-typedef QMultiMap<QString, class DBBrowserObject> objectMap;
-
-class DBBrowserObject
-{
-public:
-    DBBrowserObject() : table(""), name( "" ) { }
-    DBBrowserObject( const QString& wname,const QString& wsql, const QString& wtype, const QString& tbl_name )
-        : table(wname), name( wname), sql( wsql ), type(wtype), table_name(tbl_name)
-    { }
-
-    QString getname() const { return name; }
-    QString getsql() const { return sql; }
-    QString gettype() const { return type; }
-    QString getTableName() const { return table_name; }
-    sqlb::Table table;
-private:
-    QString name;
-    QString sql;
-    QString type;
-    QString table_name;     // The name of the table this object references, interesting for views, triggers and indices
-};
+typedef QMultiMap<QString, sqlb::ObjectPtr> objectMap;
 
 class DBBrowserDB : public QObject
 {
     Q_OBJECT
 
 public:
-    explicit DBBrowserDB () : _db(0), isEncrypted(false), isReadOnly(false) {}
+    explicit DBBrowserDB () : _db(0), isEncrypted(false), isReadOnly(false), dontCheckForStructureUpdates(false) {}
     virtual ~DBBrowserDB (){}
-    bool open ( const QString & db);
+    bool open(const QString& db, bool readOnly = false);
     bool attach(const QString& filename, QString attach_as = "");
     bool create ( const QString & db);
     bool close();
@@ -54,9 +34,10 @@ public:
     bool revertToSavepoint(const QString& pointname = "RESTOREPOINT");
     bool releaseAllSavepoints();
     bool revertAll();
-    bool dump(const QString & filename, const QStringList &tablesToDump, bool insertColNames, bool insertNew, bool exportSchema, bool exportData);
-    bool executeSQL ( const QString & statement, bool dirtyDB=true, bool logsql=true);
+    bool dump(const QString & filename, const QStringList &tablesToDump, bool insertColNames, bool insertNew, bool exportSchema, bool exportData, bool keepOldSchema);
+    bool executeSQL(QString statement, bool dirtyDB = true, bool logsql = true);
     bool executeMultiSQL(const QString& statement, bool dirty = true, bool log = false);
+    const QString& lastError() const { return lastErrorMessage; }
 
     /**
      * @brief getRow Executes a sqlite statement to get the rowdata(columns)
@@ -85,8 +66,8 @@ public:
      * @return An sqlite conform INSERT INTO statement with empty values. (NULL,'',0)
      */
     QString emptyInsertStmt(const sqlb::Table& t, const QString& pk_value = QString()) const;
-    bool deleteRecord(const QString& table, const QString& rowid);
-    bool updateRecord(const QString& table, const QString& column, const QString& rowid, const QByteArray& value, bool itsBlob);
+    bool deleteRecords(const QString& table, const QStringList& rowids);
+    bool updateRecord(const QString& table, const QString& column, const QString& rowid, const QByteArray& value, bool itsBlob, const QString& pseudo_pk = QString());
 
     bool createTable(const QString& name, const sqlb::FieldVector& structure);
     bool renameTable(const QString& from_table, const QString& to_table);
@@ -94,16 +75,17 @@ public:
 
     /**
      * @brief renameColumn Can be used to rename, modify or drop an existing column of a given table
-     * @param tablename Specifies the table name
+     * @param table_name Specifies the name of the table to edit
+     * @param table Specifies the table to edit. The table constraints are used from this but not the columns
      * @param name Name of the column to edit
      * @param to The new field definition with changed name, type or the like. If Null-Pointer is given the column is dropped.
      * @param move Set this to a value != 0 to move the new column to a different position
      * @return true if renaming was successful, false if not. In the latter case also lastErrorMessage is set
      */
-    bool renameColumn(const QString& tablename, const QString& name, sqlb::FieldPtr to, int move = 0);
+    bool renameColumn(const QString& tablename, const sqlb::Table& table, const QString& name, sqlb::FieldPtr to, int move = 0);
 
     objectMap getBrowsableObjects() const;
-    DBBrowserObject getObjectByName(const QString& name) const;
+    const sqlb::ObjectPtr getObjectByName(const QString& name) const;
     bool isOpen() const;
     bool encrypted() const { return isEncrypted; }
     bool readOnly() const { return isReadOnly; }
@@ -118,25 +100,39 @@ public:
 
     bool loadExtension(const QString& filename);
 
+    QVector<QPair<QString, QString>> queryColumnInformation(const QString& object_name);
+
+    QString generateSavepointName(const QString& identifier = QString()) const;
+
     sqlite3 * _db;
 
     objectMap objMap;
 
-    QString lastErrorMessage;
-
 signals:
     void sqlExecuted(QString sql, int msgtype);
     void dbChanged(bool dirty);
+    void structureUpdated();
 
 private:
     QString curDBFilename;
-
+    QString lastErrorMessage;
     QStringList savepointList;
-
     bool isEncrypted;
     bool isReadOnly;
 
     bool tryEncryptionSettings(const QString& filename, bool* encrypted, CipherDialog*& cipherSettings);
+
+    bool dontCheckForStructureUpdates;
+
+    class NoStructureUpdateChecks
+    {
+    public:
+        NoStructureUpdateChecks(DBBrowserDB& db) : m_db(db) { m_db.dontCheckForStructureUpdates = true; }
+        ~NoStructureUpdateChecks() { m_db.dontCheckForStructureUpdates = false; }
+
+    private:
+          DBBrowserDB& m_db;
+    };
 };
 
 #endif
