@@ -1,8 +1,8 @@
 #include "sqlitetablemodel.h"
 #include "sqlitedb.h"
 #include "sqlite.h"
+#include "Settings.h"
 
-#include "PreferencesDialog.h"
 #include <QDebug>
 #include <QMessageBox>
 #include <QApplication>
@@ -11,7 +11,7 @@
 #include <QFile>
 #include <QUrl>
 
-SqliteTableModel::SqliteTableModel(QObject* parent, DBBrowserDB* db, size_t chunkSize, const QString& encoding)
+SqliteTableModel::SqliteTableModel(DBBrowserDB& db, QObject* parent, size_t chunkSize, const QString& encoding)
     : QAbstractTableModel(parent)
     , m_db(db)
     , m_rowCount(0)
@@ -24,12 +24,15 @@ SqliteTableModel::SqliteTableModel(QObject* parent, DBBrowserDB* db, size_t chun
 
 void SqliteTableModel::reset()
 {
+    m_sTable.clear();
+    m_sRowidColumn.clear();
     m_iSortColumn = 0;
     m_sSortOrder = "ASC";
     m_headers.clear();
     m_mWhere.clear();
     m_vDataTypes.clear();
     m_vDisplayFormat.clear();
+    m_pseudoPk.clear();
 }
 
 void SqliteTableModel::setChunkSize(size_t chunksize)
@@ -37,23 +40,28 @@ void SqliteTableModel::setChunkSize(size_t chunksize)
     m_chunkSize = chunksize;
 }
 
-void SqliteTableModel::setTable(const QString& table, const QVector<QString>& display_format)
+void SqliteTableModel::setTable(const QString& table, int sortColumn, Qt::SortOrder sortOrder, const QVector<QString>& display_format)
 {
+    // Unset all previous settings. When setting a table all information on the previously browsed data set is removed first.
     reset();
 
+    // Save the other parameters
     m_sTable = table;
     m_vDisplayFormat = display_format;
 
+    // The first column is the rowid column and therefore is always of type integer
     m_vDataTypes.push_back(SQLITE_INTEGER);
 
+    // Get the data types of all other columns as well as the column names
     bool allOk = false;
-    if(m_db->getObjectByName(table).gettype() == "table")
+    if(m_db.getObjectByName(table)->type() == sqlb::Object::Types::Table)
     {
-        sqlb::Table t = sqlb::Table::parseSQL(m_db->getObjectByName(table).getsql()).first;
-        if(t.fields().size()) // parsing was OK
+        sqlb::TablePtr t = m_db.getObjectByName(table).dynamicCast<sqlb::Table>();
+        if(t && t->fields().size()) // parsing was OK
         {
-            m_headers.push_back(t.rowidColumn());
-            m_headers.append(t.fieldNames());
+            m_sRowidColumn = t->rowidColumn();
+            m_headers.push_back(m_sRowidColumn);
+            m_headers.append(t->fieldNames());
 
             // parse columns types
             static QStringList dataTypes = QStringList()
@@ -61,7 +69,7 @@ void SqliteTableModel::setTable(const QString& table, const QVector<QString>& di
                     << "REAL"
                     << "TEXT"
                     << "BLOB";
-            foreach(const sqlb::FieldPtr fld,  t.fields())
+            foreach(const sqlb::FieldPtr fld,  t->fields())
             {
                 QString name(fld->type().toUpper());
                 int colType = dataTypes.indexOf(name);
@@ -72,14 +80,22 @@ void SqliteTableModel::setTable(const QString& table, const QVector<QString>& di
         }
     }
 
+    // If for one reason or another (either it's a view or we couldn't parse the table statement) we couldn't get the field
+    // information we retrieve it from SQLite using an extra query.
+    // NOTE: It would be nice to eventually get rid of this piece here. As soon as the grammar parser is good enough...
     if(!allOk)
     {
         QString sColumnQuery = QString::fromUtf8("SELECT * FROM %1;").arg(sqlb::escapeIdentifier(table));
+        m_sRowidColumn = "rowid";
         m_headers.push_back("rowid");
         m_headers.append(getColumns(sColumnQuery, m_vDataTypes));
     }
 
-    buildQuery();
+    // Set sort parameters. We're setting the sort column to an invalid value before calling sort() because this way, in sort() the
+    // current sort order is always changed and thus buildQuery() is always going to be called.
+    // This is also why we don't need to call buildQuery() here again.
+    m_iSortColumn = -1;
+    sort(sortColumn, sortOrder);
 }
 
 namespace {
@@ -97,12 +113,12 @@ void SqliteTableModel::setQuery(const QString& sQuery, bool dontClearHeaders)
     if(!dontClearHeaders)
         reset();
 
-    if(!m_db->isOpen())
+    if(!m_db.isOpen())
         return;
 
     m_sQuery = sQuery.trimmed();
 
-    m_sQuery.remove(QRegExp("(?=\\s*[-]{2})[^'\"\n]*($|\\n)"));
+    removeCommentsFromQuery(m_sQuery);
 
     // do a count query to get the full row count in a fast manner
     m_rowCount = getQueryRowCount();
@@ -137,7 +153,7 @@ int SqliteTableModel::getQueryRowCount()
         // So just execute the statement as it is and fetch all results counting the rows
         sqlite3_stmt* stmt;
         QByteArray utf8Query = m_sQuery.toUtf8();
-        if(sqlite3_prepare_v2(m_db->_db, utf8Query, utf8Query.size(), &stmt, NULL) == SQLITE_OK)
+        if(sqlite3_prepare_v2(m_db._db, utf8Query, utf8Query.size(), &stmt, NULL) == SQLITE_OK)
         {
             retval = 0;
             while(sqlite3_step(stmt) == SQLITE_ROW)
@@ -152,11 +168,11 @@ int SqliteTableModel::getQueryRowCount()
     } else {
         // If it is a normal query - hopefully starting with SELECT - just do a COUNT on it and return the results
         QString sCountQuery = QString("SELECT COUNT(*) FROM (%1);").arg(rtrimChar(m_sQuery, ';'));
-        m_db->logSQL(sCountQuery, kLogMsg_App);
+        m_db.logSQL(sCountQuery, kLogMsg_App);
         QByteArray utf8Query = sCountQuery.toUtf8();
 
         sqlite3_stmt* stmt;
-        int status = sqlite3_prepare_v2(m_db->_db, utf8Query, utf8Query.size(), &stmt, NULL);
+        int status = sqlite3_prepare_v2(m_db._db, utf8Query, utf8Query.size(), &stmt, NULL);
         if(status == SQLITE_OK)
         {
             status = sqlite3_step(stmt);
@@ -222,11 +238,11 @@ QVariant SqliteTableModel::data(const QModelIndex &index, int role) const
 
         if(role == Qt::DisplayRole && m_data.at(index.row()).at(index.column()).isNull())
         {
-            return PreferencesDialog::getSettingsValue("databrowser", "null_text").toString();
+            return Settings::getValue("databrowser", "null_text").toString();
         } else if(role == Qt::DisplayRole && isBinary(index)) {
             return "BLOB";
         } else if(role == Qt::DisplayRole) {
-            int limit = PreferencesDialog::getSettingsValue("databrowser", "symbol_limit").toInt();
+            int limit = Settings::getValue("databrowser", "symbol_limit").toInt();
             QByteArray displayText = m_data.at(index.row()).at(index.column());
             if (displayText.length() > limit) {
                 // Add "..." to the end of truncated strings
@@ -244,16 +260,16 @@ QVariant SqliteTableModel::data(const QModelIndex &index, int role) const
         return font;
     } else if(role == Qt::ForegroundRole) {
         if(m_data.at(index.row()).at(index.column()).isNull())
-            return QColor(PreferencesDialog::getSettingsValue("databrowser", "null_fg_colour").toString());
+            return QColor(Settings::getValue("databrowser", "null_fg_colour").toString());
         else if (isBinary(index))
-            return QColor(PreferencesDialog::getSettingsValue("databrowser", "bin_fg_colour").toString());
-        return QColor(PreferencesDialog::getSettingsValue("databrowser", "reg_fg_colour").toString());
+            return QColor(Settings::getValue("databrowser", "bin_fg_colour").toString());
+        return QColor(Settings::getValue("databrowser", "reg_fg_colour").toString());
     } else if (role == Qt::BackgroundRole) {
         if(m_data.at(index.row()).at(index.column()).isNull())
-            return QColor(PreferencesDialog::getSettingsValue("databrowser", "null_bg_colour").toString());
+            return QColor(Settings::getValue("databrowser", "null_bg_colour").toString());
         else if (isBinary(index))
-            return QColor(PreferencesDialog::getSettingsValue("databrowser", "bin_bg_colour").toString());
-        return QColor(PreferencesDialog::getSettingsValue("databrowser", "reg_bg_colour").toString());
+            return QColor(Settings::getValue("databrowser", "bin_bg_colour").toString());
+        return QColor(Settings::getValue("databrowser", "reg_bg_colour").toString());
     } else if(role == Qt::ToolTipRole) {
         sqlb::ForeignKeyClause fk = getForeignKeyClause(index.column()-1);
         if(fk.isSet())
@@ -267,22 +283,41 @@ QVariant SqliteTableModel::data(const QModelIndex &index, int role) const
 
 sqlb::ForeignKeyClause SqliteTableModel::getForeignKeyClause(int column) const
 {
-    DBBrowserObject obj = m_db->getObjectByName(m_sTable);
-    if(obj.getname().size())
+    static const sqlb::ForeignKeyClause empty_foreign_key_clause;
+
+    // No foreign keys when not browsing a table. This usually happens when executing custom SQL statements
+    // and browsing the result set instead of browsing an entire table.
+    if(m_sTable.isEmpty())
+        return empty_foreign_key_clause;
+
+    // Retrieve database object and check if it is a table. If it isn't stop here and don't return a foreign
+    // key. This happens for views which don't have foreign keys (though we might want to think about how we
+    // can check for foreign keys in the underlying tables for some purposes like tool tips).
+    sqlb::ObjectPtr obj = m_db.getObjectByName(m_sTable);
+    if(obj->type() != sqlb::Object::Table)
+        return empty_foreign_key_clause;
+
+    // Convert object to a table and check if the column number is in the valid range
+    sqlb::TablePtr tbl = obj.dynamicCast<sqlb::Table>();
+    if(tbl && tbl->name().size() && (column >= 0 && column < tbl->fields().count()))
+    {
         // Note that the rowid column has number -1 here, it can safely be excluded since there will never be a
         // foreign key on that column.
-        if (column >= 0 && column < obj.table.fields().count())
-        {
-            return obj.table.fields().at(column)->foreignKey();
-        } else {
-            return sqlb::ForeignKeyClause();
-        }
-    else
-        return sqlb::ForeignKeyClause();
+
+        sqlb::ConstraintPtr ptr = tbl->constraint({tbl->fields().at(column)}, sqlb::Constraint::ForeignKeyConstraintType);
+        if(ptr)
+            return *(ptr.dynamicCast<sqlb::ForeignKeyClause>());
+    }
+
+    return empty_foreign_key_clause;
 }
 
 bool SqliteTableModel::setData(const QModelIndex& index, const QVariant& value, int role)
 {
+    // Don't even try setting any data if we're not browsing a table, i.e. the model data comes from a custom query
+    if(!isEditable())
+        return false;
+
     // This function is for in-place editing.
     // So, BLOB flag is false every times.
     return setTypedData(index, false, value, role);
@@ -295,12 +330,25 @@ bool SqliteTableModel::setTypedData(const QModelIndex& index, bool isBlob, const
         QByteArray newValue = encode(value.toByteArray());
         QByteArray oldValue = m_data.at(index.row()).at(index.column());
 
+        // Special handling for integer columns: instead of setting an integer column to an empty string, set it to '0' when it is also
+        // used in a primary key. Otherwise SQLite will always output an 'datatype mismatch' error.
+        if(newValue == "" && !newValue.isNull())
+        {
+            sqlb::TablePtr table = m_db.getObjectByName(m_sTable).dynamicCast<sqlb::Table>();
+            if(table)
+            {
+                sqlb::FieldPtr field = table->field(table->findField(m_headers.at(index.column())));
+                if(table->primaryKey().contains(field) && field->isInteger())
+                    newValue = "0";
+            }
+        }
+
         // Don't do anything if the data hasn't changed
         // To differentiate NULL and empty byte arrays, we also compare the NULL flag
         if(oldValue == newValue && oldValue.isNull() == newValue.isNull())
             return true;
 
-        if(m_db->updateRecord(m_sTable, m_headers.at(index.column()), m_data[index.row()].at(0), newValue, isBlob))
+        if(m_db.updateRecord(m_sTable, m_headers.at(index.column()), m_data[index.row()].at(0), newValue, isBlob, m_pseudoPk))
         {
             // Only update the cache if this row has already been read, if not there's no need to do any changes to the cache
             if(index.row() < m_data.size())
@@ -309,7 +357,7 @@ bool SqliteTableModel::setTypedData(const QModelIndex& index, bool isBlob, const
             emit(dataChanged(index, index));
             return true;
         } else {
-            QMessageBox::warning(0, qApp->applicationName(), tr("Error changing data:\n%1").arg(m_db->lastErrorMessage));
+            QMessageBox::warning(0, qApp->applicationName(), tr("Error changing data:\n%1").arg(m_db.lastError()));
             return false;
         }
     }
@@ -367,6 +415,9 @@ void SqliteTableModel::sort(int column, Qt::SortOrder order)
 
 bool SqliteTableModel::insertRows(int row, int count, const QModelIndex& parent)
 {
+    if(!isEditable())
+        return false;
+
     QByteArrayList blank_data;
     for(int i=0; i < m_headers.size(); ++i)
         blank_data.push_back("");
@@ -374,7 +425,7 @@ bool SqliteTableModel::insertRows(int row, int count, const QModelIndex& parent)
     DataType tempList;
     for(int i=row; i < row + count; ++i)
     {
-        QString rowid = m_db->addRecord(m_sTable);
+        QString rowid = m_db.addRecord(m_sTable);
         if(rowid.isNull())
         {
             return false;
@@ -385,7 +436,7 @@ bool SqliteTableModel::insertRows(int row, int count, const QModelIndex& parent)
 
         // update column with default values
         QByteArrayList rowdata;
-        if( m_db->getRow(m_sTable, rowid, rowdata) )
+        if( m_db.getRow(m_sTable, rowid, rowdata) )
         {
             for(int j=1; j < m_headers.size(); ++j)
             {
@@ -405,29 +456,43 @@ bool SqliteTableModel::insertRows(int row, int count, const QModelIndex& parent)
 
 bool SqliteTableModel::removeRows(int row, int count, const QModelIndex& parent)
 {
+    if(!isEditable())
+        return false;
+
     beginRemoveRows(parent, row, row + count - 1);
 
+    bool ok = true;
+
+    QStringList rowids;
     for(int i=count-1;i>=0;i--)
     {
-        m_db->deleteRecord(m_sTable, m_data.at(row + i).at(0));
+        rowids.append(m_data.at(row + i).at(0));
         m_data.removeAt(row + i);
+        --m_rowCount;
+    }
+    if(!m_db.deleteRecords(m_sTable, rowids))
+    {
+        ok = false;
     }
 
-    m_rowCount -= count;
-
     endRemoveRows();
-    return true;
+    return ok;
 }
 
 QModelIndex SqliteTableModel::dittoRecord(int old_row)
 {
+    if(!isEditable())
+        return QModelIndex();
+
+    insertRow(rowCount());
     int firstEditedColumn = 0;
     int new_row = rowCount() - 1;
 
-    sqlb::Table t = sqlb::Table::parseSQL(m_db->getObjectByName(m_sTable).getsql()).first;
+    sqlb::TablePtr t = m_db.getObjectByName(m_sTable).dynamicCast<sqlb::Table>();
 
-    for (int col = 0; col < t.fields().size(); ++col) {
-        if (!t.fields().at(col)->primaryKey()) {
+    sqlb::FieldVector pk = t->primaryKey();
+    for (int col = 0; col < t->fields().size(); ++col) {
+        if(!pk.contains(t->fields().at(col))) {
             if (!firstEditedColumn)
                 firstEditedColumn = col + 1;
 
@@ -457,10 +522,10 @@ void SqliteTableModel::fetchData(unsigned int from, unsigned to)
         else
             sLimitQuery = queryTemp + QString(" LIMIT %1, %2;").arg(from).arg(to-from);
     }
-    m_db->logSQL(sLimitQuery, kLogMsg_App);
+    m_db.logSQL(sLimitQuery, kLogMsg_App);
     QByteArray utf8Query = sLimitQuery.toUtf8();
     sqlite3_stmt *stmt;
-    int status = sqlite3_prepare_v2(m_db->_db, utf8Query, utf8Query.size(), &stmt, NULL);
+    int status = sqlite3_prepare_v2(m_db._db, utf8Query, utf8Query.size(), &stmt, NULL);
 
     if(SQLITE_OK == status)
     {
@@ -540,11 +605,70 @@ void SqliteTableModel::buildQuery()
     setQuery(sql, true);
 }
 
+void SqliteTableModel::removeCommentsFromQuery(QString& query) {
+    // first remove block comments
+    {
+        QRegExp rxSQL("^((?:(?:[^'/]|/(?![*]))*|'[^']*')*)(/[*](?:[^*]|[*](?!/))*[*]/)(.*)$");	// set up regex to find block comment
+        QString result;
+
+        while(query.size() != 0)
+        {
+            int pos = rxSQL.indexIn(query);
+            if(pos > -1)
+            {
+                result += rxSQL.cap(1) + " ";
+                query = rxSQL.cap(3);
+            } else {
+                result += query;
+                query = "";
+            }
+        }
+        query = result;
+    }
+
+    // deal with end-of-line comments
+    {
+        /* The regular expression for removing end of line comments works like this:
+         * ^((?:(?:[^'-]|-(?!-))*|(?:'[^']*'))*)(--.*)$
+         * ^                                          $ # anchor beginning and end of string so we use it all
+         *  (                                  )(    )  # two separate capture groups for code and comment
+         *                                       --.*   # comment starts with -- and consumes everything afterwards
+         *   (?:                 |           )*         # code is none or many strings alternating with non-strings
+         *                        (?:'[^']*')           # a string is a quote, followed by none or more non-quotes, followed by a quote
+         *      (?:[^'-]|-(?!-))*                       # non-string is a sequence of characters which aren't quotes or hyphens,
+         *                                                OR if they are hyphens then they can't be followed immediately by another hyphen
+         */
+        QRegExp rxSQL("^((?:(?:[^'-]|-(?!-))*|(?:'[^']*'))*)(--[^\\r\\n]*)([\\r\\n]*)(.*)$");	// set up regex to find end-of-line comment
+        QString result;
+
+        while(query.size() != 0)
+        {
+            int pos = rxSQL.indexIn(query);
+            if(pos > -1)
+            {
+                result += rxSQL.cap(1) + rxSQL.cap(3);
+                query = rxSQL.cap(4);
+            } else {
+                result += query;
+                query = "";
+            }
+        }
+
+        query = result.trimmed();
+    }
+
+    // Remove multiple line breaks that might have been created by deleting comments till the end of the line but not including the line break
+    query.replace(QRegExp("\\n+"), "\n");
+
+    // Also remove any remaining whitespace at the end of each line
+    query.replace(QRegExp("[ \t]+\n"), "\n");
+}
+
 QStringList SqliteTableModel::getColumns(const QString& sQuery, QVector<int>& fieldsTypes)
 {
     sqlite3_stmt* stmt;
     QByteArray utf8Query = sQuery.toUtf8();
-    int status = sqlite3_prepare_v2(m_db->_db, utf8Query, utf8Query.size(), &stmt, NULL);
+    int status = sqlite3_prepare_v2(m_db._db, utf8Query, utf8Query.size(), &stmt, NULL);
     QStringList listColumns;
     if(SQLITE_OK == status)
     {
@@ -565,48 +689,87 @@ void SqliteTableModel::updateFilter(int column, const QString& value)
 {
     // Check for any special comparison operators at the beginning of the value string. If there are none default to LIKE.
     QString op = "LIKE";
-    QString val;
+    QString val, val2;
     QString escape;
-    bool numeric = false;
-    if(value.left(2) == ">=" || value.left(2) == "<=" || value.left(2) == "<>")
-    {
-        bool ok;
-        value.mid(2).toFloat(&ok);
-        if(ok)
-        {
-            op = value.left(2);
-            val = value.mid(2);
-            numeric = true;
+    bool numeric = false, ok = false;
+    // range/BETWEEN operator
+    if (value.contains("~")) {
+        int sepIdx = value.indexOf('~');
+        val  = value.mid(0, sepIdx);
+        val2 = value.mid(sepIdx+1);
+        val.toFloat(&ok);
+        if (ok) {
+            val2.toFloat(&ok);
+            ok = ok && (val.toFloat() < val2.toFloat());
         }
-    } else if(value.left(1) == ">" || value.left(1) == "<") {
-        bool ok;
-        value.mid(1).toFloat(&ok);
-        if(ok)
-        {
-            op = value.left(1);
-            val = value.mid(1);
-            numeric = true;
-        }
-    } else if(value.left(1) == "=") {
-        op = "=";
-        val = value.mid(1);
+    }
+    if (ok) {
+        op = "BETWEEN";
+        numeric = true;
     } else {
-        // Keep the default LIKE operator
-
-        // Set the escape character if one has been specified in the settings dialog
-        QString escape_character = PreferencesDialog::getSettingsValue("databrowser", "filter_escape").toString();
-        if(escape_character == "'") escape_character = "''";
-        if(escape_character.length())
-            escape = QString("ESCAPE '%1'").arg(escape_character);
-
-        // Add % wildcards at the start and at the beginning of the filter query, but only if there weren't set any
-        // wildcards manually. The idea is to assume that a user who's just typing characters expects the wildcards to
-        // be added but a user who adds them herself knows what she's doing and doesn't want us to mess up her query.
-        if(!value.contains("%"))
+        val.clear();
+        val2.clear();
+        if(value.left(2) == ">=" || value.left(2) == "<=" || value.left(2) == "<>")
         {
-            val = value;
-            val.prepend('%');
-            val.append('%');
+            // Check if we're filtering for '<> NULL'. In this case we need a special comparison operator.
+            if(value.left(2) == "<>" && value.mid(2) == "NULL")
+            {
+                // We are filtering for '<> NULL'. Override the comparison operator to search for NULL values in this column. Also treat search value (NULL) as number,
+                // in order to avoid putting quotes around it.
+                op = "IS NOT";
+                numeric = true;
+                val = "NULL";
+            } else {
+                bool ok;
+                value.mid(2).toFloat(&ok);
+                if(ok)
+                {
+                    op = value.left(2);
+                    val = value.mid(2);
+                    numeric = true;
+                }
+            }
+        } else if(value.left(1) == ">" || value.left(1) == "<") {
+            bool ok;
+            value.mid(1).toFloat(&ok);
+            if(ok)
+            {
+                op = value.left(1);
+                val = value.mid(1);
+                numeric = true;
+            }
+        } else if(value.left(1) == "=") {
+            val = value.mid(1);
+
+            // Check if value to compare with is 'NULL'
+            if(val != "NULL")
+            {
+                // It's not, so just compare normally to the value, whatever it is.
+                op = "=";
+            } else {
+                // It is NULL. Override the comparison operator to search for NULL values in this column. Also treat search value (NULL) as number,
+                // in order to avoid putting quotes around it.
+                op = "IS";
+                numeric = true;
+            }
+        } else {
+            // Keep the default LIKE operator
+
+            // Set the escape character if one has been specified in the settings dialog
+            QString escape_character = Settings::getValue("databrowser", "filter_escape").toString();
+            if(escape_character == "'") escape_character = "''";
+            if(escape_character.length())
+                escape = QString("ESCAPE '%1'").arg(escape_character);
+
+            // Add % wildcards at the start and at the beginning of the filter query, but only if there weren't set any
+            // wildcards manually. The idea is to assume that a user who's just typing characters expects the wildcards to
+            // be added but a user who adds them herself knows what she's doing and doesn't want us to mess up her query.
+            if(!value.contains("%"))
+            {
+                val = value;
+                val.prepend('%');
+                val.append('%');
+            }
         }
     }
     if(val.isEmpty())
@@ -617,8 +780,13 @@ void SqliteTableModel::updateFilter(int column, const QString& value)
     // If the value was set to an empty string remove any filter for this column. Otherwise insert a new filter rule or replace the old one if there is already one
     if(val == "''" || val == "'%'" || val == "'%%'")
         m_mWhere.remove(column);
-    else
-        m_mWhere.insert(column, op + " " + QString(encode(val.toUtf8())) + " " + escape);
+    else {
+        QString whereClause(op + " " + QString(encode(val.toUtf8())));
+        if (!val2.isEmpty())
+            whereClause += " AND " + QString(encode(val2.toUtf8()));
+        whereClause += " " + escape;
+        m_mWhere.insert(column, whereClause);
+    }
 
     // Build the new query
     buildQuery();
@@ -680,3 +848,24 @@ bool SqliteTableModel::dropMimeData(const QMimeData* data, Qt::DropAction, int r
 
     return false;
 }
+
+void SqliteTableModel::setPseudoPk(const QString& pseudoPk)
+{
+    if(pseudoPk.isEmpty())
+    {
+        m_pseudoPk.clear();
+        if(m_headers.size())
+            m_headers[0] = m_sRowidColumn;
+    } else {
+        m_pseudoPk = pseudoPk;
+        if(m_headers.size())
+            m_headers[0] = pseudoPk;
+    }
+
+    buildQuery();
+}
+
+bool SqliteTableModel::isEditable() const
+{
+    return !m_sTable.isEmpty();
+}
