@@ -11,8 +11,13 @@ namespace sqlb { class ForeignKeyClause; }
 class SqliteTableModel : public QAbstractTableModel
 {
     Q_OBJECT
+
+#ifdef REGEX_UNIT_TEST
+    friend class TestRegex;
+#endif
+
 public:
-    explicit SqliteTableModel(QObject *parent = 0, DBBrowserDB* db = 0, size_t chunkSize = 50000, const QString& encoding = QString());
+    explicit SqliteTableModel(DBBrowserDB& db, QObject *parent = 0, size_t chunkSize = 50000, const QString& encoding = QString());
     void reset();
 
     int rowCount(const QModelIndex &parent = QModelIndex()) const;
@@ -33,7 +38,7 @@ public:
 
     void setQuery(const QString& sQuery, bool dontClearHeaders = false);
     QString query() const { return m_sQuery; }
-    void setTable(const QString& table, const QVector<QString> &display_format = QVector<QString>());
+    void setTable(const QString& table, int sortColumn = 0, Qt::SortOrder sortOrder = Qt::AscendingOrder, const QVector<QString> &display_format = QVector<QString>());
     void setChunkSize(size_t chunksize);
     void sort(int column, Qt::SortOrder order = Qt::AscendingOrder);
 
@@ -43,13 +48,25 @@ public:
 
     bool isBinary(const QModelIndex& index) const;
 
-    void setEncoding(QString encoding) { m_encoding = encoding; }
+    void setEncoding(const QString& encoding) { m_encoding = encoding; }
     QString encoding() const { return m_encoding; }
 
+    // The pseudo-primary key is exclusively for editing views
+    void setPseudoPk(const QString& pseudoPk);
+    QString pseudoPk() const { return m_pseudoPk; }
+
     typedef QList<QByteArray> QByteArrayList;
 
     sqlb::ForeignKeyClause getForeignKeyClause(int column) const;
 
+    // This returns true if the model is set up for editing. The model is able to operate in more or less two different modes, table browsing
+    // and query browsing. We only support editing data for the table browsing mode and not for the query mode. This function returns true if
+    // the model is currently editable, i.e. it's running in table mode.
+    bool isEditable() const;
+
+    // Helper function for removing all comments from a SQL query
+    static void removeCommentsFromQuery(QString& query);
+
 public slots:
     void updateFilter(int column, const QString& value);
 
@@ -68,7 +85,7 @@ private:
     QByteArray encode(const QByteArray& str) const;
     QByteArray decode(const QByteArray& str) const;
 
-    DBBrowserDB* m_db;
+    DBBrowserDB& m_db;
     int m_rowCount;
     QStringList m_headers;
     typedef QList<QByteArrayList> DataType;
@@ -76,6 +93,8 @@ private:
 
     QString m_sQuery;
     QString m_sTable;
+    QString m_sRowidColumn;
+    QString m_pseudoPk;
     int m_iSortColumn;
     QString m_sSortOrder;
     QMap<int, QString> m_mWhere;
