@@ -12,6 +12,7 @@ private slots:
     void notnull();
     void withoutRowid();
     void foreignKeys();
+    void uniqueConstraint();
 
     void parseSQL();
     void parseSQLdefaultexpr();
@@ -24,6 +25,7 @@ private slots:
     void parseSQLEscapedQuotes();
     void parseSQLForeignKeys();
     void parseSQLCheckConstraint();
+    void parseDefaultValues();
     void createTableWithIn();
     void createTableWithNotLikeConstraint();
 };
